import java.util.*;
public class Dijkstra {
    Vector<Node> nodes;
    Vector<Node> targets;
    boolean sorted ;
    public Dijkstra() {
	nodes = new Vector<Node>();
	sorted = false;
	targets = null;
    }
    public void addEdge(int src,int dst,int weight) {
	Node s = getNode(src);
	Node d = getNode(dst);
	s.addEdge(d,weight);
    }
    protected Node searchNode(int id) {
	return (sorted) ? Node.binarySearch(id,nodes): Node.linearSearch(id,nodes);
    }
    protected Node getNode(int id) {
	Node node = searchNode(id);
	if (node != null) return node;
	sorted = false;
	node = new Node(id);
	nodes.add(node);
	return node;
    }
    public void addDestination(int dst) {
	if (targets == null) targets = new Vector<Node>();
	targets.add(getNode(dst));
    }
    protected boolean allFound() {
	if (targets == null) return false;
	for (int i=0; i<targets.size(); i++) {
	    Node t = targets.elementAt(i);
	    if (! t.isFixed()) return false;
	}
	//System.out.println("trim tree");
	return true;
    }
    public void solve(int src) {
	if (!sorted) {Node.sort(nodes); sorted = true; }
	Node s = getNode(src);
	//System.out.println(this);
	s.setWeight(0);
	while (!allFound()) {
	    Node node = searchUnfixedMinNode();
	    //System.out.println("min:"+node);
	    if (node == null) return; // all nodes are connected
	    if (node.getWeight() == Integer.MAX_VALUE) return; // unconnected
	    node.setFixed();
	    Vector<Edge> edges = node.edges;
	    int w = node.getWeight();
	    for (int i=0; i<edges.size(); i++) {
		Edge e = edges.elementAt(i);
		Node dst = e.dst;
		int nw = w + e.weight;
		if (nw < dst.getWeight()) {
		    dst.setWeight(nw);
		    dst.setVia(node);
		}
	    }
	}
    }
    protected Node searchUnfixedMinNode() {
	Node ret = null;
	for (int i=0; i<nodes.size(); i++) {
	    Node node = nodes.elementAt(i);
	    //System.out.println("searchUnfixedMinNode:"+node);
	    if (node.isFixed()) continue;
	    if (ret == null) { ret = node; continue; }
	    if (ret.getWeight() > node.getWeight()) ret = node;
	}
	return ret;
    }
    public int getWeight(int nid) {
	Node node = searchNode(nid);
	if (node == null) return Integer.MAX_VALUE;
	return node.getWeight();
    }
    public int[] getPath(int nid) {
	Vector<Node> v = new Vector<Node>();
	for (Node node = searchNode(nid); node != null; node=node.getVia())
	    v.add(node);
	int[] a = new int[v.size()];
	for (int i=0; i<a.length; i++)
	    a[a.length-1-i] = v.elementAt(i).getId();
	return a;
    }
    public String toString() {
	StringBuffer sb = new StringBuffer();
	for (int i=0; i<nodes.size(); i++) {
	    sb.append(nodes.elementAt(i));
	}
	return sb.toString();
    }
}
